/****************************************
****  pacifi3d.c  - Main Source File  ***
*****************************************/

//-- Include Files -----------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "SDL.h"
#include <ctype.h>
#include <string.h>

#include "pacifi3d.h"
#include "z80/Z80.h"
#include "icon.h"
#include "rom.h"

#define FPS 60.606060
#define REFRESHTIME 1000/FPS
#define Z80_VBL_CYCLES 3072000/FPS

//-- Private variables -------------------------------------------------------
static Z80 z80;
static Uint32 next;

//-- Function Prototypes -----------------------------------------------------
void pacman_reset(void);
void pacman_run(void);
void pacman_shutdown(void);

//----------------------------------------------------------------------------
int main(int argc, char* argv[])
{
    SDL_RWops *iconfp;
    SDL_Surface *icon_surface;

    /* Initialise SDL */
    if((SDL_Init(SDL_INIT_JOYSTICK)==-1)) {
        printf("Could not initialize SDL: %s.\n", SDL_GetError());
        exit(-1);
    }

    /* Displays version number, date and time */
    puts(VERSION1);
    puts(VERSION2);

    /* Register exit procedure */
    atexit(pacman_shutdown);

    /* memory init */
    memory_init();

    /* Load ROMS */
    rom_load("roms/namcopac.6e",0x0000,0x1000,memory);
    rom_load("roms/namcopac.6f",0x1000,0x1000,memory);
    rom_load("roms/namcopac.6h",0x2000,0x1000,memory);
    rom_load("roms/namcopac.6j",0x3000,0x1000,memory);

    rom_load("roms/pacman.5e",0x0000,0x1000,tilerom);
    rom_load("roms/pacman.5f",0x0000,0x1000,spriterom);

    rom_load("roms/82s123.7f",0x0000,0x0020,prom);
    rom_load("roms/82s126.4a",0x0020,0x0100,prom);

    /* Video init */
    video_init();

    /* set up window icon */
    iconfp=SDL_RWFromMem((void*)icon, sizeof(icon));
    icon_surface=SDL_LoadBMP_RW(iconfp,0);
    SDL_WM_SetIcon(icon_surface,NULL);

    /* set window title */
    SDL_WM_SetCaption(VERSION1,VERSION1);

    /* Initialise input */
    input_init();

    /* reset hardware */
    pacman_reset();

    /* get time for speed throttle */
    next=SDL_GetTicks()+REFRESHTIME;

    /* Enter Z80 loop */
    z80.IAutoReset=1;
    z80.IPeriod=Z80_VBL_CYCLES;
    RunZ80(&z80);

    return 0;
}


//----------------------------------------------------------------------------
void pacman_reset(void)
{
    ResetZ80(&z80);
}

//----------------------------------------------------------------------------
void pacman_shutdown(void)
{
    // Close everything and free memory
    input_shutdown();
    video_shutdown();

    printf("pacman: System Shutdown.\n");

    SDL_Quit();
}


//----------------------------------------------------------------------------
/* z80 loop */
/* This procedure is called every Z80_VBL_CYCLES by the Z80 emulation */
word LoopZ80(register Z80 *R)
{
    Uint32 now;

    /* Call display routine */
    video_draw_screen();

    /* Update keys and Joystick */
    processEvents();

    /* Speed Throttle */
    now=SDL_GetTicks();
    if (now < next) {
        SDL_Delay(next-now);
    }
    next+=REFRESHTIME;

    /* raise IRQ */
    return INT_IRQ;
}


